Object.defineProperty(exports, '__esModule', { value: true });

var sugarHigh = require('sugar-high');
var unistUtilMap = require('unist-util-map');
var rangeParser = require('parse-numeric-range');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var rangeParser__default = /*#__PURE__*/_interopDefault(rangeParser);

const parseLang = (str)=>{
    const match = (regexp)=>{
        const m = (str || '').match(regexp);
        return Array.isArray(m) ? m.filter(Boolean) : [];
    };
    const [lang = 'unknown'] = match(/^[a-zA-Z\d-]*/g);
    const range = rangeParser__default.default(match(/\{(.*?)\}$/g).join(',').replace(/^\{/, '').replace(/\}$/, ''));
    return {
        lang,
        range
    };
};
const h = (type, attrs, children)=>{
    return {
        type: 'element',
        tagName: type,
        data: {
            hName: type,
            hProperties: attrs,
            hChildren: children
        },
        properties: attrs,
        children
    };
};
const highlight = ()=>(tree)=>{
        return unistUtilMap.map(tree, (node)=>{
            const { type, tagName } = node;
            if (tagName !== 'code' && type !== 'code') return node;
            const { lang } = parseLang(node.lang);
            const codeText = node.value || node.children.filter(({ type })=>type === 'text').map(({ value })=>value).pop();
            const childrenLines = sugarHigh.generate(sugarHigh.tokenize(codeText));
            for (const line of childrenLines){
                line.children.push(h('span', {
                    className: 'sh__token--line'
                }, [
                    {
                        type: 'text',
                        value: '\n'
                    }
                ]));
            }
            const code = h('code', {
                className: `sh-lang--${lang}`,
                ['data-sh-language']: `${lang}`
            }, childrenLines);
            const pre = h('pre', {
                className: `sh-lang--${lang}`
            }, [
                code
            ]);
            return pre;
        });
    };

exports.default = highlight;
exports.highlight = highlight;
